

function [Xhat_tnsr,  Relerrs2] = TWFProj2D(Y, Params)

NormX        =    getGlobalNormX;
Masks2       =    getGlobalMasks;
X_Vdeo       =    getGlobalX_Vdeo;
Xhat_tnsr    =    zeros(Params.n1, Params.n2, Params.q);

%   Initialization

for   na    =   1   :   Params.q
    
    Yk      =   Y(:,:,:,na);
    Xhat_k  =   randn(Params.n1, Params.n2);
    Xhat_k  =   Xhat_k / norm(Xhat_k, 'fro');    % Initial guess
    normest =   sqrt(sum(Yk(:)) / numel(Yk(:)));    % Estimate norm to scale eigenvector
    
    A       =   @(I)  fft2(conj(Masks2(:,:,:,na)) .* reshape(repmat(I, [1 Params.L]), size(I, 1), size(I, 2), Params.L));
    At      =   @(W)  sum(Masks2(:,:,:,na) .* ifft2(W), 3) * size(W, 1) * size(W, 2);
    
    for   tt    =   1   :   Params.npower_iter,                     % Truncated power iterations
        
        Yk_tr   =   Yk .* (abs(Yk) <= Params.alpha_y^2 * normest^2 );
        Xhat_k  =   At(Yk_tr .* (A(Xhat_k)));
        Xhat_k  =   Xhat_k / norm(Xhat_k,'fro');
        
    end
    
    Xhat_k                 =     normest * Xhat_k;                   % Apply scaling
    Xhat_tnsr(:,:,na)      =     Xhat_k;
    
end

%   Projection

Xhat            =   reshape(Xhat_tnsr, Params.n1 * Params.n2, Params.q);
[Uhat,~,~,~ ]   =   BlockIter(Xhat, Params.MaxIter, Params.r);
Xhat            =   Uhat * (Uhat' * Xhat);
Xhat_tnsr       =   reshape(Xhat, Params.n1, Params.n2, Params.q);

Relerrs         =   zeros(Params.q, Params.T);

%       Main Loop

for   nt   =   1   :   Params.T
    
    fprintf('TWFP Iteration # %d of main loop...\n', nt);
    
    for   aa   =   1  :  Params.q
        
        Masks_k     =   Masks2(:,:,:,aa);
        A           =   @(I)  fft2(conj(Masks_k) .* reshape(repmat(I, [1 Params.L]), size(I, 1), size(I, 2), Params.L));
        At          =   @(W) sum(Masks_k .* ifft2(W), 3) * size(W,1) * size(W,2);
        
        grad        =   compute_grad(Xhat_tnsr(:,:,aa), Y(:,:,:,aa), Params, A, At);
        Xhat_tnsr(:,:,aa)   =   Xhat_tnsr(:,:,aa) - Params.mu * grad;
    end
    
    Xhat            =   reshape(Xhat_tnsr, Params.n1 * Params.n2, Params.q);
    [Uhat,~,~,~]    =   BlockIter(Xhat, Params.MaxIter, Params.r);
    Xhat            =   Uhat * (Uhat' * Xhat);
    Xhat_tnsr       =   reshape(Xhat, Params.n1, Params.n2, Params.q);
    for   na   =   1  :  Params.q
       z       =   Xhat_tnsr(:,:,na);
       x       =   X_Vdeo(:,:,na);
       Relerrs(na,nt)  =   norm(x - exp(-1i*angle(trace(x' * z))) * z, 'fro')^2;
    end
    
end

Relerrs2    =    sum(Relerrs)/NormX;


