

function [Xhat_tnsr, ERR]  =  alt_min_phs_rtv_2D(Y, Params)

%///////////////////////////////////////////////////////////////////
%     Getting global values

X_Vdeo       =       getGlobalX_Vdeo;
NormX        =       getGlobalNormX;
Masks2       =       getGlobalMasks;

%      Function handles

Afull 	     =	   @(I) fft2(conj(Masks2).*reshape(repmat(I,[1,Params.L]), Params.n_1, Params.n_2, Params.L, Params.q));
Afull_t      =	   @(E) sum(sum( Masks2 .* ifft2(E), 3) , 4)* Params.n_1 * Params.n_2; %* size(E,3);
Afull_tk     =     @(S) sum( Masks2 .* ifft2(S), 3)*Params.n_1*Params.n_2; % * size(E,1) * size(E,2) * size(E,3);

%///////////////////////////////////////////////////////////////////
%      Truncating measurements  

tic;
Ytrk     =   zeros(Params.n_1,Params.n_2,Params.L,Params.q);
for  ni  =   1   : Params.q
    Yk       =   reshape(Y(:,:,:,ni), Params.n_1 * Params.n_2 * Params.L, 1);
    normest  =   sum(Yk(:))/Params.m;
    Eyk      =   (Yk <= Params.alpha_y^2*normest);
    Ytrk(:,:,:,ni)    =   reshape(Eyk.*Yk,Params.n_1,Params.n_2,Params.L);
end

%//////////////////////////////////////////////////////////////////
%      Block power method for estimating initial U

fprintf('Initialization begins...\n');

U_hat           =   randn(Params.n_1*Params.n_2,Params.r);
[U_hat, ~, ~]   =   qr(U_hat, 0);
Uhat_tnsr       =   reshape(U_hat, Params.n_1, Params.n_2, Params.r);

for   nt   =    1  : Params.itr_num_pow_mth
    
    for    nr   =  1 : Params.r
         Uhat_tnsr(:,:,nr)   =   Afull_t( Ytrk .* Afull(repmat(Uhat_tnsr(:,:,nr), [1, 1, Params.q])));
    end
    
    [Uhat, ~, ~]    =   qr(reshape(Uhat_tnsr, Params.n_1 * Params.n_2, Params.r), 0);
    Uhat_tnsr       =   reshape(Uhat, Params.n_1, Params.n_2, Params.r);
    
end


%////////////////////////////////////////////////////////////////
%  Finding the initial estimate of B

AUnew    =   zeros(Params.n_1*Params.n_2*Params.L,Params.q,Params.r);
Ybk      =   zeros(Params.n_1,Params.n_2,Params.r,Params.q);

for  nr  =   1  :  Params.r
    
    AUnew(:,:,nr)   =  reshape(Afull(repmat(Uhat_tnsr(:,:,nr),[1,1,Params.q])), Params.n_1*Params.n_2*Params.L,Params.q);
    Ybk(:,:,nr,:)   =  Afull_tk(Y.* Afull(repmat(Uhat_tnsr(:,:,nr), [1,1,Params.q])));
    
end

Bhat     =   zeros(Params.r,Params.q);
for  na  =   1  :  Params.q
    
    ybk           =   Uhat' * reshape(Ybk(:,:,:,na), Params.n_1 * Params.n_2, Params.r);
    [b_t,~,~]     =   svd(ybk);
    nb            =   sqrt(sum(reshape(Y(:,:,:,na), Params.n_1 * Params.n_2 * Params.L,1)) / Params.m);
    nc            =   Params.m*Params.r /(norm(reshape(AUnew(:,na,:), Params.n_1 * Params.n_2 * Params.L, Params.r), 'fro')^2);
    norm_b_a_hat  =   sqrt(nc) * nb;
    Bhat(:,na)    =   norm_b_a_hat * b_t(:,1);
    
end

%%
%////////////////////////////////////////////////////////////////////////
%           Main Loop
%///////////////////////////////////////////////////////////////////////

Xhat        =   Uhat * Bhat;
Xhat_tnsr   =   reshape(Xhat, Params.n_1, Params.n_2, Params.q);
Chat        =   exp(1i * angle(Afull(Xhat_tnsr))); % Initial phase
Ysqrt       =   sqrt(Y);

Rel_Err     =   zeros(Params.q, Params.T_altmin);

for  t   =  1  :  Params.T_altmin
    
    fprintf('AMT iteration # %d ... \n', t)
    %//////////////////////////////////////////////////////////////////////
    %                Updating Uhat
    %//////////////////////////////////////////////////////////////////////
    
    K1       =   Chat .* Ysqrt;
    Zvec     =   reshape(K1, Params.n_1 * Params.n_2 * Params.L * Params.q, 1);
    
    
    [Uhat_tnsr,~,~,~]   =   cgls_new(@mult_H2, @mult_Ht2 , Zvec, 0, 1e-6 , 3);
    Uhat                =   reshape(Uhat_tnsr, Params.n_1 * Params.n_2, Params.r);
    
    %//////////////////////////////////////////////////////////////////////
    %               Updating Bhat and Chat
    %/////////////////////////////////////////////////////////////////////
    
    %K        =   reshape(K1,Params.n_1*Params.n_2*Params.L*Params.q,1);
    K2        =   reshape(K1, Params.n_1 * Params.n_2 * Params.L, Params.q);
    
    Uhat_tnsr=    reshape(Uhat, Params.n_1, Params.n_2, Params.r);
    
    R1        =   zeros(Params.n_1, Params.n_2, Params.L, Params.q, Params.r);
    Bhat      =   zeros(Params.r, Params.q);
    for   nr  =   1  :  Params.r
        R1(:,:,:,:,nr)  =   Afull(repmat(Uhat_tnsr(:,:,nr), [1, 1, Params.q]));
    end
    for   aa  =   1  :  Params.q
        R           =   reshape(R1(:,:,:,aa, :), Params.n_1 * Params.n_2 * Params.L, Params.r);
        K           =   reshape(K2(:,aa), Params.n_1 * Params.n_2 * Params.L, 1);
        Bhat(:,aa)  =   mldivide(R , K); %  Bhat(:,aa)   =  R \K3;
    end
    
    %//////////////////////////////////////////////////////////////////////
    %                   Updating Chat
    %//////////////////////////////////////////////////////////////////////
    Xhat        =   Uhat * Bhat;
    Xhat_tnsr   =   reshape(Xhat, Params.n_1, Params.n_2, Params.q);
    Chat        =   exp(1i * angle(Afull(Xhat_tnsr)));  
    
    %/////////////////////////////////////////////////////
    %          Calculating error 
    
    Tmp_Err_X   =   zeros(Params.q, 1);
    
         for     na     =   1    :     Params.q
            xa_hat      =   Xhat_tnsr(:,:,na);
            xa          =   X_Vdeo(:,:,na);
            Tmp_Err_X(na) 	=   norm(xa - exp(-1i * angle(trace(xa' * xa_hat))) * xa_hat, 'fro')^2;
         end
 
          Rel_Err(:, t)  =   Tmp_Err_X;
         
    
end
      
ERR           =    sum(Rel_Err)/NormX;

%%

%///////////////////////////////////////////////////////////////////////
%                   Defining Functions
%///////////////////////////////////////////////////////////////////////

%   Defining mult_H

    function  i_out  =  mult_H2(i_in)
        I_mat   =    reshape(i_in,(Params.n_1*Params.n_2), Params.r);
        % i_out    =   zeros(Params.q*Params.m, 1);
        Xmat    =     I_mat * Bhat;
        Xmat2   =     reshape(Xmat,Params.n_1,Params.n_2,Params.q);
        Iout    =     Afull(Xmat2);
        i_out   =     reshape(Iout,Params.q*Params.m, 1);
    end

%   Defining mult_Ht

    function  w_out  =  mult_Ht2(w_in)
        w_out     =    zeros(Params.n_1*Params.n_2*Params.r, 1);
        TmpVec    =    permute(Afull_tk(reshape(w_in, Params.n_1,Params.n_2,Params.L,Params.q)), [1,2,4,3]);
        for   nk  =  1 : Params.q
            w_out    =   w_out + kron(Bhat(:,nk), reshape(TmpVec(:,:,nk), Params.n_1*Params.n_2, 1));
        end
        
    end
%/////////////////////////////////////////////////////////////////////////

end
